@@ -46,9 +46,9 @@
 #include "gsm-xsmp-server.h"
 #include "gsm-store.h"
 
-#define GSM_GCONF_DEFAULT_SESSION_KEY           "/desktop/gnome/session/default-session"
-#define GSM_GCONF_REQUIRED_COMPONENTS_DIRECTORY "/desktop/gnome/session/required-components"
-#define GSM_GCONF_REQUIRED_COMPONENTS_KEY       "/desktop/gnome/session/required-components"
+#define GSM_GCONF_DEFAULT_SESSION_KEY           "/desktop/gnome/session/default_session"
+#define GSM_GCONF_REQUIRED_COMPONENTS_DIRECTORY "/desktop/gnome/session/required_components"
+#define GSM_GCONF_REQUIRED_COMPONENTS_LIST_KEY  "/desktop/gnome/session/required_components_list"
 
 #define GSM_DBUS_NAME "org.gnome.SessionManager"
 
@@ -247,6 +247,8 @@ find_desktop_file_for_app_name (const char *name,
         return app_path;
 }
 
+/* This doesn't contain the required components, so we need to always
+ * call append_required_apps() after a call to append_default_apps(). */
 static void
 append_default_apps (GsmManager *manager,
                      const char *default_session_key,
@@ -312,11 +314,16 @@ append_required_apps (GsmManager *manager)
         GSList      *r;
         GConfClient *client;
 
+        g_debug ("main: *** Adding required apps");
+
         client = gconf_client_get_default ();
         required_components = gconf_client_get_list (client,
-                                                     GSM_GCONF_REQUIRED_COMPONENTS_KEY,
+                                                     GSM_GCONF_REQUIRED_COMPONENTS_LIST_KEY,
                                                      GCONF_VALUE_STRING,
                                                      NULL);
+        if (required_components == NULL) {
+                g_warning ("No required applications specified");
+        }
 
         for (r = required_components; r != NULL; r = r->next) {
                 char       *path;
@@ -334,6 +341,7 @@ append_required_apps (GsmManager *manager)
                                         component);
 
                 default_provider = gconf_client_get_string (client, path, NULL);
+                g_debug ("main: %s looking for component: '%s'", path, default_provider);
                 if (default_provider != NULL) {
                         char *app_path;
 
@@ -352,6 +360,8 @@ append_required_apps (GsmManager *manager)
                 g_free (path);
         }
 
+        g_debug ("main: *** Done adding required apps");
+
         g_slist_foreach (required_components, (GFunc)g_free, NULL);
         g_slist_free (required_components);
 
@@ -379,11 +389,11 @@ load_standard_apps (GsmManager *manager,
 
         append_saved_session_apps (manager);
 
-        /* We don't do this in the failsafe case, because the default
-         * session should include all requirements anyway. */
+ out:
+        /* We do this at the end in case a saved session contains an
+         * application that already provides one of the components. */
         append_required_apps (manager);
 
- out:
         g_strfreev (autostart_dirs);
 }
 
